#!/usr/bin/env perl
#
# Copyright 2012 Dimitrios - Georgios Kontopoulos
#    <dgkontopoulos[at]member[dot]fsf[dot]org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License, as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details at
#  <http://www.gnu.org/licenses>.

use strict;
use warnings;

use Date::Calc qw(Add_Delta_Days Delta_Days);
use DBI;
use Encode;
use Gtk2 -init;
use Gtk2::SimpleList;
use Locale::gettext;
use POSIX;
use utf8;

my $d = Locale::gettext->domain('OrthCal');
$d->dir('/opt/extras.ubuntu.com/orthcal/translations/locale/');

if ( $ARGV[0] eq "about" )
{
    my $window = Gtk2::Window->new('toplevel');
    $window->set_default_icon_from_file(
        "/opt/extras.ubuntu.com/orthcal/content/images/OrthCal.png");
    $window->set_title( $d->get("About OrthCal") );
    $window->signal_connect( destroy => sub { Gtk2->main_quit() } );
    $window->set_resizable(0);

    my $top_info = $d->get(<<'ABOUT');
<b>OrthCal, the Eastern Orthodox Calendar</b>, v. 1.2
		        <a href="http://launchpad.net/orthcal">Homepage</a> - - <a href="https://www.transifex.com/projects/p/OrthCal">Translations</a>
         (C) 2012 <a href="mailto:dgkontopoulos@member.fsf.org?Subject=OrthCal">Dimitrios - Georgios Kontopoulos</a>
ABOUT
    my $second_info = $d->get(<<'ABOUT');
<span size="small">This application was written for the <a href="http://developer.ubuntu.com/showdown/">Ubuntu App Showdown contest</a> of
July 2012, mainly as practice.

The indicator's code is written in <a href="http://www.python.org/">Python</a>/<a href="http://www.pygtk.org/">PyGTK</a>, whereas the main part
of the calendar is written in <a href="http://www.perl.org/">Perl</a>/<a href="http://search.cpan.org/~xaoc/Gtk2-1.244/lib/Gtk2.pm">GTK2</a>.</span>
ABOUT
    my $acknowledgements = $d->get(<<'ABOUT');
<span size="small"><b><u>Acknowledgements:</u></b>
	➜The daily feasts have been extracted from <a href="http://orthodoxwiki.org/">OrthodoxWiki</a> (CC BY-
SA 2.5) and <a href="http://www.saint.gr">saint.gr</a> (<i>"freely ye have received, freely give"</i>).
	➜The background picture of a cross has been borrowed under the
CC BY-NC 3.0 license from <a href="http://www.fotopedia.com/items/scientik-zLHqNZYLcIs">Petr Jan Juračka</a>.
	➜The application's icon has been borrowed under the CC BY-NC-SA
2.0 license from <a href="http://www.flickr.com/photos/avangardphoto/6715331853/">Avangard Photography</a>.</span>
ABOUT
    my $license = $d->get(<<'ABOUT');
<span size="small"><b><u>License:</u></b>
<i>This program is free software; you can redistribute it and/or modify it
under the terms of the <a href="http://www.gnu.org/licenses/gpl.html">GNU General Public License, as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version</a>.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.</i>

Images and content mentioned in the above section are offered under
their original respective licenses.</span>
ABOUT

    chomp $top_info;
    my $label1 = Gtk2::Label->new();
    $label1->set_markup($top_info);
    my $separator1 = Gtk2::HSeparator->new;

    chomp $second_info;
    my $label2 = Gtk2::Label->new();
    $label2->set_markup($second_info);
    my $separator2 = Gtk2::HSeparator->new;

    chomp $acknowledgements;
    my $label3 = Gtk2::Label->new();
    $label3->set_markup($acknowledgements);
    my $separator3 = Gtk2::HSeparator->new;

    chomp $license;
    my $label4 = Gtk2::Label->new();
    $label4->set_markup($license);

    my $vbox = Gtk2::VBox->new( '0', '10' );
    $vbox->add($label1);
    $vbox->add($separator1);
    $vbox->add($label2);
    $vbox->add($separator2);
    $vbox->add($label3);
    $vbox->add($separator3);
    $vbox->add($label4);

    $window->set_border_width(15);
    $window->set_position('mouse');
    $window->add($vbox);

    my $back_pixbuf = Gtk2::Gdk::Pixbuf->new_from_file(
        "/opt/extras.ubuntu.com/orthcal/content/images/bg.jpg");
    my ( $pixmap, $mask ) = $back_pixbuf->render_pixmap_and_mask(255);
    my $style = $window->get_style();
    $style = $style->copy();
    $style->bg_pixmap( "normal", $pixmap );
    $window->set_style($style);
    $window->show_all;
    Gtk2->main;
    exit;
}
elsif ( $ARGV[0] eq "options" )
{
    my $window = Gtk2::Window->new('toplevel');
    $window->set_default_icon_from_file(
        "/opt/extras.ubuntu.com/orthcal/content/images/OrthCal.png");
    $window->signal_connect( destroy => sub { Gtk2->main_quit() } );
    $window->set_resizable(0);

    my $column1 = $d->get('Daily Feasts source');
    my $column2 = $d->get('Language');
    my $column3 = $d->get('Completed');

    my $slist = Gtk2::SimpleList->new(
        $column1 => 'text',
        $column2 => 'text',
        $column3 => 'text'
    );

    @{ $slist->{data} } = (
        [ 'orthodoxwiki.org', 'English',          "\t+" ],
        [ 'saint.gr',         'Ελληνικά', "\t+" ]
    );
    my $username = getpwuid($<);
    my $source;
    if ( -e "/home/$username/.config/orthcal/source" )
    {
        open my $source_fh, '<', "/home/$username/.config/orthcal/source";
        $source = <$source_fh>;
        close $source_fh;
        chomp $source;
        if ( $source =~ /orthodoxwiki[.]org/ )
        {
            $slist->select(0);
        }
        elsif ( $source =~ /saint[.]gr/ )
        {
            $slist->select(1);
        }
        else
        {
            $slist->select(0);
            open my $source_fh, '>', "/home/$username/.config/orthcal/source";
            print {$source_fh} 'orthodoxwiki.org';
            close $source_fh;
        }
    }
    else
    {
        unless ( -d "/home/$username/.config/orthcal/" )
        {
            system "mkdir -p /home/$username/.config/orthcal/";
        }
        open my $source_fh, '>', "/home/$username/.config/orthcal/source";
        print {$source_fh} 'orthodoxwiki.org';
        close $source_fh;
    }
    $slist->signal_connect( row_activated => \&row_clicked, $username );
    my $list_message = $d->get(<<'END');
<b>Double click</b> to change from one source to another.
END
    chomp $list_message;

    my $label = Gtk2::Label->new();
    $label->set_markup($list_message);

    my $separator = Gtk2::HSeparator->new;

    my $autostart = $d->get("Start OrthCal on system startup.");
    my $checkbox  = Gtk2::CheckButton->new($autostart);

    if ( -e "/home/$username/.config/autostart/orthcal.desktop" )
    {
        $checkbox->set_active(1);
    }
    else
    {
        $checkbox->set_active(0);
    }

    $checkbox->signal_connect( toggled => \&autostart, $username );

    my $vbox = Gtk2::VBox->new( '0', '10' );
    $vbox->add($slist);
    $vbox->add($label);
    $vbox->add($separator);
    $vbox->add($checkbox);

    $window->set_title( $d->get("Options for OrthCal") );
    $window->set_border_width(15);
    $window->set_position('mouse');
    $window->add($vbox);

    $window->show_all;
    Gtk2->main;
    exit;
}

my $date = `date +%d/%m/%Y`;
my ( $day, $month, $year );

if ( $date =~ /(\d+)\/(\d+)\/(\d+)/ )
{
    $day   = $1;
    $month = $2;
    $year  = $3;
}

if ( $ARGV[0] eq "Julian" )
{
    ( $year, $month, $day ) = Add_Delta_Days( $year, $month, $day, '-13' );
    if ( $month !~ /\d\d/ )
    {
        $month = '0' . $month;
    }
    if ( $day !~ /\d\d/ )
    {
        $day = '0' . $day;
    }
}
my $username = getpwuid($<);
my $source;
if ( -e "/home/$username/.config/orthcal/source" )
{
    open my $source_fh, '<', "/home/$username/.config/orthcal/source";
    $source = <$source_fh>;
    close $source_fh;
    chomp $source;
}
else
{
    $source = 'orthodoxwiki.org';
}
my $dbh =
  DBI->connect( "dbi:SQLite:/opt/extras.ubuntu.com/orthcal/content/saints.db",
    "", "" );
my $db_sel = $dbh->prepare(
"SELECT * FROM Saints WHERE Month = $month AND Day = $day AND source LIKE '%$source%'"
);
$db_sel->execute();
my $row    = $db_sel->fetch;
my $feasts = $row->[0];
$feasts = decode "UTF8", $feasts;

my ( $Easterd, $Easterm, $Eastery );
&OrthEaster;

( $Eastery, $Easterm, $Easterd ) =
  Add_Delta_Days( $Eastery, $Easterm, $Easterd, '13' );

my $Dd = Delta_Days( $Eastery, $Easterm, $Easterd, $year, $month, $day );

my %movable;
$movable{'-77'} = $d->get('<b>Zacchaeus</b> Sunday');
$movable{'-70'} = $d->get('Publican & Pharisee Sunday');
$movable{'-63'} = $d->get('Sunday of the Prodigal Son');
$movable{'-56'} = $d->get('Meat Fare Sunday (Sunday of Last Judgement)');
$movable{'-49'} =
  $d->get('Cheesefare (Forgiveness) Sunday / Expulsion from Paradise');
$movable{'-48'} = $d->get('Great Lent begins');
$movable{'-42'} = $d->get('Sunday of Orthodoxy');
$movable{'-35'} = $d->get('Sunday of St. <b>Gregory</b> Palamas');
$movable{'-28'} = $d->get('Sunday of the Holy Cross');
$movable{'-21'} = $d->get('Sunday of St. <b>John</b> Climacus');
$movable{'-14'} = $d->get('Sunday of St. <b>Mary</b> of Egypt');
$movable{'-8'}  = $d->get('<b>Lazarus</b> Saturday');
$movable{'-7'}  = $d->get('Palm Sunday');
$movable{'-6'}  = $d->get('Holy Monday');
$movable{'-5'}  = $d->get('Holy Tuesday');
$movable{'-4'}  = $d->get('Holy Wednesday');
$movable{'-3'}  = $d->get('Holy Thursday');
$movable{'-2'}  = $d->get('Holy Friday');
$movable{'-1'}  = $d->get('Holy Saturday');
$movable{'0'}   = $d->get('Pascha');
$movable{'7'}   = $d->get('Sunday of St. <b>Thomas</b>');
$movable{'14'}  = $d->get('Sunday of Myrrh-bearing Women');
$movable{'21'}  = $d->get('Sunday of the Paralytic');
$movable{'24'}  = $d->get('Midfeast of Pentecost');
$movable{'28'}  = $d->get('Sunday of the Samaritan Woman');
$movable{'35'}  = $d->get('Sunday of the Blind Man');
$movable{'38'}  = $d->get('Leavetaking of Pascha');
$movable{'39'}  = $d->get('Ascension Thursday');
$movable{'42'}  = $d->get('Fathers of the 1st Ecumenical Council');
$movable{'49'}  = $d->get('Pentecost Sunday');
$movable{'50'}  = $d->get('Day of the Holy Spirit');
$movable{'56'}  = $d->get('Sunday of All Saints');
$movable{'63'}  = $d->get('All Saints of North America');

binmode STDOUT, ":utf8";

if ( $ARGV[0] eq "Julian" )
{
    $Dd += 13;
}

if ( defined $movable{$Dd} )
{
    $feasts = '✙✙ <b>' . $movable{$Dd} . '</b> ✙✙✙' . "\n" . $feasts;
}
$feasts = '✙ ' . $feasts;
$feasts =~ s/\n(\w|<|")/\n\✙ $1/g;
my $occurences = ( $feasts =~ tr/\n// );

if ( $occurences < 26 )
{
    if ( $occurences > 20 )
    {
        $feasts =~ s/\n(.+\n.+\n.+\n.+)\n(.+)/\n$1\n\n$2/g;
    }
    elsif ( $occurences > 15 )
    {
        $feasts =~ s/\n(.+\n.+\n.+)\n(.+)/\n$1\n\n$2/g;
    }
    elsif ( $occurences > 10 )
    {
        $feasts =~ s/\n(.+\n.+)\n(.+)/\n$1\n\n$2/g;
    }
    elsif ( $occurences > 5 )
    {
        $feasts =~ s/\n(.+)\n(.+)/\n$1\n\n$2/g;
    }
    else
    {
        $feasts =~ s/\n(.+)/\n\n$1/g;
    }
}
elsif ( $occurences < 36 )
{
    chomp $feasts;
    $feasts = '<span size="x-small">' . $feasts . '</span>';
}
else
{
    chomp $feasts;
    $feasts = '<span size="xx-small">' . $feasts . '</span>';
}

$feasts =~ s/\n$//;

my $window = Gtk2::Window->new('toplevel');
$window->set_default_icon_from_file(
    "/opt/extras.ubuntu.com/orthcal/content/images/OrthCal.png");
$window->signal_connect( destroy => sub { Gtk2->main_quit() } );
$window->set_resizable(0);

if ( $ARGV[0] eq "Julian" )
{
    my ( $year2, $month2, $day2 );
    ( $year2, $month2, $day2 ) = Add_Delta_Days( $year, $month, $day, '13' );
    $window->set_title(
        "$day/$month/$year // [$day2/$month2/$year2] -- OrthCal");
}
else
{
    $window->set_title("$day/$month/$year -- OrthCal");
}

my $label = Gtk2::Label->new();
$label->set_markup($feasts);
$label->set_line_wrap('TRUE');
$window->set_border_width(15);
$window->set_position('mouse');
$window->add($label);

my $back_pixbuf = Gtk2::Gdk::Pixbuf->new_from_file(
    "/opt/extras.ubuntu.com/orthcal/content/images/bg.jpg");
my ( $pixmap, $mask ) = $back_pixbuf->render_pixmap_and_mask(255);
my $style = $window->get_style();
$style = $style->copy();
$style->bg_pixmap( "normal", $pixmap );
$window->set_style($style);
$window->show_all;
Gtk2->main;

sub autostart
{
    my $self     = shift;
    my $username = shift;

    if ( $self->get_active )
    {
        unless ( -d "/home/$username/.config/autostart/" )
        {
            system "mkdir -p /home/$username/.config/autostart/";
        }
        open my $desktop_fh, '>',
          "/home/$username/.config/autostart/orthcal.desktop";
        print {$desktop_fh} <<'END';

[Desktop Entry]
Type=Application
Exec=/opt/extras.ubuntu.com/orthcal/bin/OrthCal_ind.py
Hidden=false
NoDisplay=false
X-GNOME-Autostart-enabled=true
Name=OrthCal
END
        close $desktop_fh;
    }
    else
    {
        if ( -e "/home/$username/.config/autostart/orthcal.desktop" )
        {
            unlink "/home/$username/.config/autostart/orthcal.desktop";
        }
    }
}

sub row_clicked
{
    my $self     = shift;
    my $username = $_[2];
    if ( $self->get_selection )
    {
        my $selection = $self->get_selection;
        $selection->selected_foreach(
            sub {
                my ( $model, $path, $iter ) = @_;
                my $value = $model->get( $iter, 0 );
                open my $source_fh, '>',
                  "/home/$username/.config/orthcal/source";
                print {$source_fh} $value;
                close $source_fh;
            }
        );
    }
}

sub OrthEaster
{
    my ( $a, $b, $c, $d, $e );

    $a       = $year % 4;
    $b       = $year % 7;
    $c       = $year % 19;
    $d       = ( 19 * $c + 15 ) % 30;
    $e       = ( 2 * $a + 4 * $b - $d + 34 ) % 7;
    $Easterm = ( ( $d + $e + 114 ) / 31 );
    $Easterd = ( ( $d + $e + 114 ) % 31 ) + 1;
    $Eastery = $year;

    return $Easterd, $Easterm, $Eastery;
}
